package uoc.tfg.model.marketdata.lightstreamer;

import java.io.Serializable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.lightstreamer.ls_client.ConnectionInfo;
import com.lightstreamer.ls_client.ConnectionListener;
import com.lightstreamer.ls_client.ExtendedTableInfo;
import com.lightstreamer.ls_client.HandyTableListener;
import com.lightstreamer.ls_client.LSClient;
import com.lightstreamer.ls_client.PushConnException;
import com.lightstreamer.ls_client.PushServerException;
import com.lightstreamer.ls_client.PushUserException;
import com.lightstreamer.ls_client.SubscrException;
import com.lightstreamer.ls_client.UpdateInfo;



public class DataFeederLS implements Serializable {
       
	private static final long serialVersionUID = 2115899296020212148L;
	final private ConnectionInfo cInfo;
    final private ExtendedTableInfo tInfo;
    final private LSClient client;
    final private FeedHandlerLS feedHandler;
    
    private int phase = 0;
    
    private final ExecutorService connectionThread;
    
	
	/**
	 * Constructor method(s)
	 */
	public DataFeederLS(AdapterInfoLS info, IDataPullerLS dataPuller) {
		this.cInfo = info.getcInfo();
        this.tInfo = info.gettInfo();
        this.client = new LSClient();
        this.feedHandler = new FeedHandlerLS(dataPuller);
        this.connectionThread = Executors.newSingleThreadExecutor();
	}
	    
    public synchronized void start(int ph) {
        if (ph != this.phase) {
            //we ignore old calls
            return;
        }
        this.start();
    }
    
    public synchronized void start() {
        //this method starts a connection effort
        this.phase++;
        connectionThread.execute(new ConnectionThread(this.phase));
    }
    
	public synchronized void pause() {
		
	}

	public synchronized void resume() {
		
	}

	public synchronized void stop() {
		changeStatus(this.phase, ConnectionStatusLS.DISCONNECTED.getValue());
	}
    
    //notification of a change in the status of the connection
    private synchronized void changeStatus(int ph, final int status) {
        if (ph != this.phase) {
            //we ignore old calls
            return;
        }
        
    }
    
    private synchronized void execute(int ph) {
        if (ph != this.phase) {
             return;
        }
        this.phase++;
        this.connect();
        this.subscribe();
    }
    
    private synchronized void connect() {
        boolean connected = false;
        //this method will not exit until the openConnection returns without throwing an exception
        while (!connected) {
            try {
                this.phase++;
                client.openConnection(this.cInfo, new StatusListener(this.phase));
                connected = true;
            } catch (PushConnException e) {
            } catch (PushServerException e) {
            } catch (PushUserException e) {
            }
            
            if (!connected) {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                }
            }
         }
    }

    private synchronized void subscribe() {
        //this method will try just one subscription.
        //we know that when this method executes we should be already connected
        //If we're not or we disconnect while subscribing we don't have to do anything here as an
        //event will be (or was) sent to the ConnectionListener that will handle the case.
        //If we're connected but the subscription fails we can't do anything as the same subscription 
        //would fail again and again (btw this should never happen)
                
        try { 
            client.subscribeTable(tInfo, new UpdateTableListener(this.phase), false);
        } catch (SubscrException e) {
        } catch (PushServerException e) {
        } catch (PushUserException e) {
        } catch (PushConnException e) {
        }
    }
    
    
    private class ConnectionThread extends Thread {
        private final int ph;

        public ConnectionThread(int ph) {
            this.ph = ph;
        }

        public void run() {
            execute(this.ph);
        }
    }
    
    private class StatusListener implements ConnectionListener {
        
        private int ph;
        private boolean isPolling;
        
        public StatusListener(int ph) {
            this.ph = ph;
        }
        
        //synchronization is useless in this class as there is only one thread calling method on this listener
        
        private void onDisconnection() {
            changeStatus(this.ph, ConnectionStatusLS.DISCONNECTED.getValue());
            start(this.ph);
        }
        
        private void onConnection() {
            if (this.isPolling) {
                changeStatus(this.ph, ConnectionStatusLS.POLLING.getValue());
            } else {
                changeStatus(this.ph, ConnectionStatusLS.STREAMING.getValue());
            }
        }
        
        @Override
        public void onActivityWarning(boolean warningOn) {
            if (warningOn) {
                changeStatus(this.ph, ConnectionStatusLS.STALLED.getValue());
            } else {
                this.onConnection();
            }
        }
    
        @Override
        public void onClose() {
            this.onDisconnection();
        }
    
        @Override
        public void onConnectionEstablished() {
        }
    
        @Override
        public void onDataError(PushServerException arg0) {
        }
    
        @Override
        public void onEnd(int arg0) {
            this.onDisconnection();
        }
    
        @Override
        public void onFailure(PushServerException arg0) {
            this.onDisconnection();
        }
    
        @Override
        public void onFailure(PushConnException arg0) {
            this.onDisconnection();
        }
    
        @Override
        public void onNewBytes(long bytes) {
        }
    
        @Override
        public void onSessionStarted(boolean isPolling) {
            this.isPolling = isPolling;
            
            this.onConnection();
            
        }

    }
    
    private class UpdateTableListener implements HandyTableListener, Serializable {
    	
		private static final long serialVersionUID = 9148566634930224648L;
		private int ph;

        public UpdateTableListener(int ph) {
        	this.ph = ph;
            feedHandler.start();
        }
        
        @Override
        public void onRawUpdatesLost(int itemIndex, String itemName, int lostUpdates) {
            return;
        }
    
        @Override
        public void onSnapshotEnd(int itemIndex, String itemName) {
            return;
        }
    
        @Override
        public void onUnsubscr(int itemIndex, String itemName) {
            return;
        }
    
        @Override
        public void onUnsubscrAll() {
            return;
        }
    
        @Override
        public synchronized void onUpdate(int itemIndex, String itemName, UpdateInfo update) {
        	if (phase != this.ph){
                return;
            }
        	
        	if(update != null)
        		feedHandler.updateQueue(update);
        		
        }
        
    }

	/**
	 * @return the feedHandler
	 */
	public FeedHandlerLS getFeedHandler() {
		return feedHandler;
	}


}
