package uoc.tfg.model.news.webcrawler;

import java.io.IOException;
import java.net.URL;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Observable;
import java.util.Queue;
import java.util.Set;
import java.util.concurrent.ConcurrentLinkedDeque;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.lucene.document.Document;
import org.apache.tika.exception.TikaException;
import org.xml.sax.SAXException;

import uoc.tfg.exception.WebCrawlingException;
import uoc.tfg.model.Status;
import uoc.tfg.model.news.IWebContentPuller;
import uoc.tfg.model.news.WebContent;

/**
 * TFG-UOC
 * 
 * @author Javier Sanzol
 * 
 * 02/12/2013
 *
 */


/*
 *
 */
public class WebCrawler extends Observable implements IWebCrawler {
	
	private IWebCrawlingStrategy strategy;
	private Status status = Status.STOPPED;
	private int numVisitedUrls = 0;
	private Set<String> visitedUrls = new HashSet<String>();
	private IWebContentPuller contentPuller;
	private List<WebContent> allEntries;
	private Queue<WebContent> lastEntries;
	private ExecutorService executor;
	
	public WebCrawler(IWebCrawlingStrategy strategy, IWebContentPuller contentPuller) {
		super();
		this.strategy = strategy;
		this.contentPuller = contentPuller;
		this.allEntries = new ArrayList<WebContent>();
		this.lastEntries = new ConcurrentLinkedDeque<WebContent>();
		this.addObserver(this.contentPuller);
		this.executor = Executors.newSingleThreadScheduledExecutor();
	}

	
	/* (non-Javadoc)
	 * @see IWebCrawler#startWebCrawler()
	 */
	@Override
	public synchronized void startWebCrawler() throws WebCrawlingException {
		if(this.status.getValue() == Status.STOPPED.getValue()) {
			this.setStatus(Status.RUNNING);
			this.executor.execute(this);
		} else {
			throw new WebCrawlingException(WebCrawlingException.CRAWLER_RUNNING);
		}
	}

	/* (non-Javadoc)
	 * @see IWebCrawler#pauseWebCrawler()
	 */
	@Override
	public synchronized void pauseWebCrawler() {
		this.setStatus(Status.PAUSED);		
	}

	/* (non-Javadoc)
	 * @see IWebCrawler#resumeWebCrawler()
	 */
	@Override
	public synchronized void resumeWebCrawler() {
		this.setStatus(Status.RUNNING);
	}

	/* (non-Javadoc)
	 * @see IWebCrawler#stopWebCrawler()
	 */
	@Override
	public synchronized void stopWebCrawler() {
		this.setStatus(Status.STOPPED);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run() {
		
		while(this.status.isnot(Status.STOPPED)) {			
			WebCrawlingURL url = strategy.getNextUrl();
			System.out.println(url.getUrlText()+"  "+url.getDepth());
			if((url != null)) {	
				this.addVisitedUrl(url.getUrlText());
				WebContent content = null;
				try {
					content = this.strategy.getContentParser().getWebContent(url.getUrl());
				} catch (IOException | ParseException e1) {}
				if(content != null) {
					int p = this.strategy.getContentParser().getPriority(content);
					if(p != 0) {
						Document doc = null;
						try {
							doc = this.strategy.getContentParser().getDocument(content.getUrl());
						} catch (IOException | SAXException | TikaException | ParseException e1) {}
						try {
							this.contentPuller.update(content);
							this.addEntry(content);
							this.contentPuller.update(doc);
						} catch (Exception e1) {}
					}
					List<URL> urls = null;
					try {
						urls = this.strategy.getContentParser().findLinksInContent(url.getUrl());
					} catch (IOException e) {}
					for(URL u: urls) {
						if(!this.visitedUrls.contains(u.toString()) && 
								url.getDepth() < this.strategy.getMaxDepth() && 
								this.IsOfInterest(u.toString()))
							this.strategy.getSeed().getUrls().add(new WebCrawlingURL(u, p, url.getDepth()+1));
					}
				}
			}
			
			while(this.status.is(Status.PAUSED))	{
				try {
					Thread.sleep(this.strategy.getDelay());
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			
			try {
				Thread.sleep(this.strategy.getDelay());
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			this.checkExit();
			
		}
		
	}
	
	private boolean IsOfInterest(String string) {
		if(string.matches("(.)*[Ii][Bb][Ee][Xx](.)*")) return true;
		else return false;
	}


	private void addVisitedUrl(String url) {

		this.visitedUrls.add(url);
		this.numVisitedUrls = this.numVisitedUrls + 1;
		this.setChanged();this.notifyObservers();
	}


	/**
	 * @param link
	 * @return
	 */
	private boolean urlIsVisitable(URL url) {
		// TODO Auto-generated method stub
		return false;
	}


	public synchronized void checkExit() {
		if(strategy.exitCriteria(this)) this.setStatus(Status.STOPPED);
	}

	/* (non-Javadoc)
	 * @see IWebCrawler#getName()
	 */
	@Override
	public String getName() {
		return this.strategy.getStrategyName();
	}

	/* (non-Javadoc)
	 * @see IWebCrawler#getStatus()
	 */
	@Override
	public Status getStatus() {
		return this.status;
	}

	/* (non-Javadoc)
	 * @see IWebCrawler#setStatus(model.Status)
	 */
	@Override
	public synchronized void setStatus(Status status) {
		this.status = status;
		this.setChanged();
		this.notifyObservers();
	}
	
	/**
	 * @return the numVisitedUrls
	 */
	@Override
	public int getNumVisitedUrls() {
		return numVisitedUrls;
	}


	/**
	 * @param numVisitedUrls the numVisitedUrls to set
	 */
	@Override
	public void setNumVisitedUrls(int numVisitedUrls) {
		this.numVisitedUrls = numVisitedUrls;
	}


	/**
	 * 
	 * @return List<WebEntry>
	 */
	@Override
	public List<WebContent> getAllEntries() {
		return allEntries;
	}


	/**
	 * @param WebCrawler
	 */
	public void setAllEntries(List<WebContent> allEntries) {
		this.allEntries = allEntries;
	}


	/**
	 * 
	 * @return Queue<WebEntry>
	 */
	@Override
	public Queue<WebContent> getLastEntries() {
		return lastEntries;
	}


	/**
	 * @param WebCrawler
	 */
	public void setLastEntries(Queue<WebContent> lastEntries) {
		this.lastEntries = lastEntries;
	}
	
	/**
	 * @param entry
	 */
	private synchronized void addEntry(WebContent entry) {
		
		this.allEntries.add(entry);
		this.lastEntries.add(entry);
		this.setChanged();
		this.notifyObservers();
	}

	
	

}
